@@ -25,7 +25,7 @@ Mail::SpamAssassin - Spam detector and markup engine
   my $status = $spamtest->check($mail);
 
   if ($status->is_spam()) {
-    $mail = $status->rewrite_mail();
+    $message = $status->rewrite_mail();
   }
   else {
     ...
@@ -94,20 +94,20 @@ use vars qw{
   @site_rules_path
 };
 
-$VERSION = "3.001000";      # update after release (same format as perl $])
-$IS_DEVEL_BUILD = 0;        # change for release versions
+$VERSION = "3.001003";      # update after release (same format as perl $])
+#$IS_DEVEL_BUILD = 1;        # change for release versions
 
 @ISA = qw();
 
 # SUB_VERSION is now just <yyyy>-<mm>-<dd>
-$SUB_VERSION = (split(/\s+/,'$LastChangedDate: 2005-09-13 19:05:10 -0700 (Tue, 13 Sep 2005) $ updated by SVN'))[1];
+$SUB_VERSION = (split(/\s+/,'$LastChangedDate: 2006-06-01 07:35:36 -0700 (Thu, 01 Jun 2006) $ updated by SVN'))[1];
 
 # If you hacked up your SA, you should add a version_tag to your .cf files.
 # This variable should not be modified directly.
 @EXTRA_VERSION = qw();
 if (defined $IS_DEVEL_BUILD && $IS_DEVEL_BUILD) {
   push(@EXTRA_VERSION,
-       ('r' . qw{$LastChangedRevision: 280740 $ updated by SVN}[1]));
+       ('r' . qw{$LastChangedRevision: 410869 $ updated by SVN}[1]));
 }
 
 sub Version {
@@ -122,6 +122,7 @@ $HOME_URL = "http://spamassassin.apache.org/";
 # version installed.  Unless you can come up with a fix for this that
 # allows "make test" to work, don't change this.
 @default_rules_path = (
+  '__local_state_dir__/spamassassin/__version__',
   '__def_rules_dir__',
   '__prefix__/share/spamassassin',
   '/usr/local/share/spamassassin',
@@ -185,7 +186,7 @@ debugging facilities are enabled.
 
 =item rules_filename
 
-The filename to load spam-identifying rules from. (optional)
+The filename/directory to load spam-identifying rules from. (optional)
 
 =item site_rules_filename
 
@@ -1319,16 +1320,24 @@ sub init {
     if ($siterules) {
       $self->{config_text} .= $self->read_pre($siterules, 'site rules pre files');
     }
+    else {
+      warn "config: could not find site rules directory\n";
+    }
+
     if ($sysrules) {
       $self->{config_text} .= $self->read_pre($sysrules, 'sys rules pre files');
     }
+    else {
+      warn "config: could not find sys rules directory\n";
+    }
 
     $fname = $sysrules;
     if ($fname) {
       $self->{config_text} .= $self->read_cf ($fname, 'default rules dir');
-      if (-f "$fname/languages") {
-	$self->{languages_filename} = "$fname/languages";
-      }
+    }
+
+    if (!$self->{languages_filename}) {
+      $self->{languages_filename} = $self->find_rule_support_file("languages");
     }
 
     $fname = $siterules;
@@ -1343,8 +1352,12 @@ sub init {
       $fname = $self->{userprefs_filename};
       $fname ||= $self->first_existing_path (@default_userprefs_path);
 
-      if (defined $fname) {
-        if (!-f $fname && !$self->{dont_copy_prefs} && !$self->create_default_prefs($fname)) {
+      if (!$self->{dont_copy_prefs}) {
+        # bug 4932: if the userprefs path doesn't exist, we need to make it, so
+        # just use the last entry in the array as the default path.
+        $fname ||= $self->sed_path($default_userprefs_path[-1]);
+
+	if (!-f $fname && !$self->create_default_prefs($fname)) {
           warn "config: failed to create default user preference file $fname\n";
         }
       }
@@ -1459,17 +1472,45 @@ sub get_and_create_userstate_dir {
 
   $fname ||= $self->first_existing_path (@default_userstate_dir);
 
-  if (defined $fname && !$self->{dont_copy_prefs}) {
+  # bug 4932: use the last default_userstate_dir entry if none of the others
+  # already exist
+  $fname ||= $self->sed_path($default_userstate_dir[-1]);
+
+  if (!$self->{dont_copy_prefs}) {
     dbg("config: using \"$fname\" for user state dir");
   }
 
+  # bug 4932: we always want to make the userstate directory, even if
+  # dont_copy_prefs is true for things like bayes, awl, etc.
   if (!-d $fname) {
     # not being able to create the *dir* is not worth a warning at all times
     eval { mkpath($fname, 0, 0700) } or dbg("config: mkdir $fname failed: $@ $!\n");
   }
+
   $fname;
 }
 
+=item $fullpath = $f->find_rule_support_file ($filename)
+
+Find a rule-support file, such as C<languages> or C<triplets.txt>,
+in the system-wide rules directory, and return its full path if
+it exists, or undef if it doesn't exist.
+
+(This API was added in SpamAssassin 3.1.1.)
+
+=cut
+
+sub find_rule_support_file {
+  my ($self, $filename) = @_;
+
+  # take a copy to avoid modifying the real one (stupid map { } side-effect)
+  my @paths = @default_rules_path;
+  return $self->first_existing_path (map {
+      s/$/\/${filename}/;
+      $_;
+    } @paths);
+}
+
 =item $f->create_default_prefs ($filename, $username [ , $userdir ] )
 
 Copy default preferences file into home directory for later use and
@@ -1501,7 +1542,7 @@ sub create_default_prefs {
     # copy in the default one for later editing
     my $defprefs = $self->first_existing_path (@Mail::SpamAssassin::default_prefs_path);
 
-    if (open (IN, "<$defprefs")) {
+    if (defined $defprefs && open (IN, "<$defprefs")) {
       $fname = Mail::SpamAssassin::Util::untaint_file_path($fname);
       if (open (OUT, ">$fname")) {
         while (<IN>) {
@@ -1524,9 +1565,12 @@ sub create_default_prefs {
         warn "config: cannot write to $fname: $!\n";
       }
     }
-    else {
+    elsif (defined $defprefs) {
       warn "config: cannot open $defprefs: $!\n";
     }
+    else {
+      warn "config: can not determine default prefs path\n";
+    }
   }
 
   return(0);
@@ -1566,9 +1610,11 @@ sub sed_path {
   my $orig_path = $path;
 
   $path =~ s/__local_rules_dir__/$self->{LOCAL_RULES_DIR} || ''/ges;
+  $path =~ s/__local_state_dir__/$self->{LOCAL_STATE_DIR} || ''/ges;
   $path =~ s/__def_rules_dir__/$self->{DEF_RULES_DIR} || ''/ges;
   $path =~ s{__prefix__}{$self->{PREFIX} || $Config{prefix} || '/usr'}ges;
-  $path =~ s{__userstate__}{$self->get_and_create_userstate_dir()}ges;
+  $path =~ s{__userstate__}{$self->get_and_create_userstate_dir() || ''}ges;
+  $path =~ s/__version__/${VERSION}/gs;
   $path =~ s/^\~([^\/]*)/$self->expand_name($1)/es;
 
   $path = Mail::SpamAssassin::Util::untaint_file_path ($path);
@@ -1583,7 +1629,7 @@ sub first_existing_path {
     $path = $self->sed_path ($p);
     if (defined $path && -e $path) { return $path; }
   }
-  $path;
+  return;
 }
 
 sub get_cf_files_in_dir {
